const kBaseEditorStyleSheet = "chrome://editor/content/EditorOverride.css";
var MOZILLA1_3_PLUS = false;

var Editor = function (editorElement, htmlEditor, textEditor) {
	this.init(editorElement, htmlEditor, textEditor);
}

Editor.prototype = {
  normalMode  : 1,
  sourceMode  : 3,
  statusMsg   : "",
  gFrameNum   : 0,
  editMode    : 1,

  init : function(editorElement, htmlEditor, textEditor) // constructor
  {
    this.editorElement = document.getElementById(editorElement);
    this.richEditor    = document.getElementById(htmlEditor);
    this.textEditor    = document.getElementById(textEditor);
    this.titleBox      = document.getElementById("doctitle");
    this.summaryBox    = document.getElementById("docDescription");
    this.categoryBox   = document.getElementById("categoryBox");
    this.editMode      = this.normalMode;
    this._loadRichEditor();
    this._loadTextEditor();
  },

  viewHtml : function()
  {
    if (this.isNormalMode() == false) { // show html
	  var textContent = this.content;
	  this.editorElement.setAttribute('selectedIndex', '0');
          this.editMode = this.normalMode;
	  this.content = textContent;
	} else {
          document.getElementById('htmlView').setAttribute("checked", true);
          document.getElementById('statusView').setAttribute("checked", false);
	  this.editorElement.setAttribute('selectedIndex', '0');
	}
        if (this.textEditor.disabled == true)
          SetDocumentEditable(false);
	this.focus();
  },

  viewSource : function()
  {
	if (this.isNormalMode() == true) { // show source
	  var textContent = this.content;
	  this.editorElement.setAttribute('selectedIndex','1');
          this.editMode = this.sourceMode;
	  this.content = textContent;
	} else {
	  this.editorElement.setAttribute('selectedIndex','1');
	}
	this.focus();
  },

  viewMetadata : function()
  {
    var currView = document.getElementById('discussEditorType');
    if (currView.getAttribute('selectedIndex') != 3) {
      currView.setAttribute('selectedIndex', 3);
      this.populateCategory();
    }
  },

  viewStatus : function()
  {
    var currView = document.getElementById('discussEditorType');
    if (currView.getAttribute('selectedIndex') != 4) {
      document.getElementById("htmlView").setAttribute("checked", false);
      document.getElementById("statusView").setAttribute("checked", true);
      currView.setAttribute('selectedIndex', 4);
    }
  },

  viewPrevPostBox : function()
  {
    var currView = document.getElementById('discussEditorType');
    if (currView.getAttribute('selectedIndex') != 5) {
      document.getElementById("htmlView").setAttribute("checked", false);
      document.getElementById("statusView").setAttribute("checked", true);
      currView.setAttribute('selectedIndex', 5);
    }
  },

  updateMetadata : function()
  {
    var docTitle = this.titleBox;
    var docDesc = this.summaryBox
    if (!docTitle.value || docTitle.value == "")
      docTitle.value = this.generateSummary(this.content, 100);
    if (!docDesc.value || docDesc.value == "")
      docDesc.value  = this.generateSummary(this.content, 500);
  },

  clearMeta : function()
  {
    this.titleBox.value = "";
    this.summaryBox.value = "";
    this.categoryBox.clearSelection();
  },

  populateCategory : function()
  {
    var ds = rdfStore.getSource("profile");
    var categoryNS = document.getElementById("blogIDs").selectedItem.value;
    this.categoryBox.setAttribute("ref", categoryNS + "#category");
    this.categoryBox.database.AddDataSource(ds);
    this.categoryBox.builder.rebuild();
    ds = null;
  },

  generateSummary : function(tmpContent, length)
  {
    var whitespace = /\s+/gi;
    var htmlTag = /<[^<>]*>/gi;
    tmpContent = tmpContent.replace(whitespace, " ");
    tmpContent = tmpContent.replace(htmlTag, "");
    tmpContent = tmpContent.slice(0, length);
    var lastFullStopIndex = tmpContent.lastIndexOf(".");
    if (lastFullStopIndex > 0)
      tmpContent = tmpContent.slice(0,lastFullStopIndex+1);
    return tmpContent;
  },

  isNormalMode : function() // check if its in html mode
  {
    if (this.editMode == this.normalMode)
      return true;
    else
      return false;
  },

  get content()  // get the content of the editor
  {
    var content = "";

    //Added by P.L.
    var flags = 0;

    if (MOZILLA1_3_PLUS == false) {
          flags = 8 + 256; // 8: OutputBodyOnly - 256: OutputEncodeEntities;  Added by P.L. - START - (Taken from chrome://composer/editor/editor.js)
          //Added 8: OutputBodyOnly for Moz1.7
	  content = this.isNormalMode() ?
                    this.editorShell.GetContentsAs("text/html", flags) :
                    //this.editorShell.editorDocument :
                    this.textEditor.value;
    } else {
      flags = 8 + 65536; // 8: OutputBodyOnly - 65536: OutputEncodeHTMLEntities  Added by P.L.  22-03-03
      //Added 8: OutputBodyOnly for Moz1.7
      var editor = this._getCurrentEditor();
      content = this.isNormalMode() ?
                editor.outputToString("text/html", flags) :
                this.textEditor.value;
    }
    if (this.isNormalMode()) {
        //content = content.replace(/^<.+<body[^<>]*>/gi, "");
        content = content.replace(/<body[^<>]*>/gi, "");  //Changed for Moz1.7 caus it is adding a space before <body>
        content = content.replace(/<\/body><\/html>/gi, " ");
        content = content.replace(/\s+/gi, " ");
    }

    return content;
  },

  set content(contentText)	// set the content for the editor
  {
	if (this.isNormalMode() == true) { // it's graphical editor
		var editorContent = '<html><head></head><body>' + contentText + "</body></html>";
		// bug <br> inserted if it's the last focused element, use \n hack to get around it
            try {
                this._getCurrentEditor().rebuildDocumentFromSource(editorContent);
            } catch (e) {
                this.editorShell.RebuildDocumentFromSource(editorContent);
            }
	} else {
		this.textEditor.value = contentText;
		this.textEditor.setAttribute('hidden','true');  //Changed from this.textEditor.setAttribute(...);htmlView
		this.textEditor.setAttribute('hidden','false');
	}
    this.updateMetadata();
  },

  dumpText : function()
  {
	alert(this.editorShell.GetContentsAs('text/plain', 2));
  },

  currentStatus : function(statusMessage, percentage)
  {
    if (percentage) {
      document.getElementById("blogStatusBar").setAttribute("mode", "determined");
      document.getElementById("blogStatusBar").setAttribute("value", percentage);
    } else {
      document.getElementById("blogStatusBar").setAttribute("mode", "undetermined");
    }
    this.statusMsg += statusMessage;
    document.getElementById("blogStatusReport").value = this.statusMsg;
  },

  focus : function()
  {
    if (this.isNormalMode())
      window.frames[this.gFrameNum].focus();
    else
      this.textEditor.focus();
  },

  ///////////////////////////////////////////////////
  // private functions
  ///////////////////////////////////////////////////
  _loadRichEditor : function ()
  {
	// setup the rich editor
    try {
      this.editorShell = this.richEditor.editorShell;
    } catch (e) {}

    if (this.editorShell) {
      dump ("Loading editorShell editor");
      window.editorShell = this.editorShell; // for external dependancies like image upload
	  this.editorShell.editorType = 'html';

      // find the window instance bottombar is in
	  for (frameNum = 0; frameNum < window.length; frameNum++)
	    if (window.frames[frameNum].name == this.richEditor.getAttribute('name')) {
		  this.editorShell.webShellWindow = window.frames[frameNum];
		  this.editorShell.contentWindow = window.frames[frameNum];
          this.gFrameNum = frameNum;
	    }

	  this.editorShell.contentsMIMEType = "text/html";
          //this.editorShell.SetDocumentCharacterSet = "UTF-8";
	  this.editorShell.RegisterDocumentStateListener(DocumentStateListener);
	  this.editorShell.LoadUrl('about:blank');
    } else {
      dump ("Loading new improved xbl editor");
      const nsIWebNavigation = Components.interfaces.nsIWebNavigation;
      try {
        var commandManager = this.richEditor.commandManager;
        commandManager.addCommandObserver(moz13_DocumentStateListener, "obs_documentCreated");
      } catch (e) { dump(" commandManager failed: "+e+"\n"); }

      try {
        this.richEditor.webNavigation.loadURI("about:blank", // uri string
             nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE, null, null, null);
      } catch (e) { dump(" EditorLoadUrl failed: "+e+"\n"); }
      MOZILLA1_3_PLUS = true;
    }
  }, // _loadRichEditor

  _loadTextEditor : function ()
  {
	var sourceWordwrap = null;
	try {	 // setup the editor is it will wrap if defined in preference
	  sourceWordwrap = discussPref.getBoolPref('discuss.sourceWordwrap');
	} catch (ex) {
          sourceWordwrap = false;
	}
	if (sourceWordwrap) {
	  this.textEditor.setAttribute('cols','1');
          this.summaryBox.setAttribute('cols','1');
	} else {
	  this.textEditor.removeAttribute('cols');
          this.summaryBox.removeAttribute('cols');
	}
  }, // _loadTextEditor

  _getCurrentEditor : function()
  {
    var editor;
    if (MOZILLA1_3_PLUS == false) {
      try {
        editor = this.editorShell.editor.QueryInterface(Components.interfaces.nsIHTMLEditor);
      } catch (e) { dump (e)+"\n"; }
    } else {
      try {
        var editorElement = this.richEditor;
        editor = editorElement.getEditor(editorElement.contentWindow);
        editor instanceof Components.interfaces.nsIEditor;
        editor instanceof Components.interfaces.nsIPlaintextEditor;
        editor instanceof Components.interfaces.nsIHTMLEditor;
      } catch (e) { dump (e)+"\n"; }
    }
    return editor;
  }
};

var DocumentStateListener =
{
  NotifyDocumentCreated: function()
  {
    //postListBox.selectIndex(0)
    dump("Editor Document created");
    setDefaultView();
    var editor = editorHandle._getCurrentEditor();
    try {
      editor.QueryInterface(Components.interfaces.nsIEditorStyleSheets);
      editor.addOverrideStyleSheet(kBaseEditorStyleSheet);
    } catch (e) { dump(e)+"Stylesheet failed to load" }
	addEditorListener();
  },
  NotifyDocumentWillBeDestroyed: function() {},
  NotifyDocumentStateChanged:function( isNowDirty )
  {
    window.updateCommands("create");
    window.updateCommands("save");
  }
};

var moz13_DocumentStateListener =
{
  observe: function(aSubject, aTopic, aData)
  {
    var commandManager = editorHandle.richEditor.commandManager;
    if (commandManager != aSubject)
      return;
    switch(aTopic) {
      case "obs_documentCreated":
        //postListBox.selectIndex(0);
        setDefaultView();
        var editor = editorHandle._getCurrentEditor();;
        try {
          editor.QueryInterface(Components.interfaces.nsIEditorStyleSheets);
          editor.addOverrideStyleSheet(kBaseEditorStyleSheet);
          editor.addEditActionListener(DocumentActionListener);
        } catch (e) { dump(e)+"Stylesheet failed to load" }
        break;
      default : break;
    }
  }
}

// Potential future code for selection handling

function addEditorListener()
{
	var editor = editorHandle.editorShell.editor.QueryInterface(Components.interfaces.nsIEditor);
	editor.addEditActionListener(DocumentActionListener);
}

var selectListener = {
	notifySelectionChanged: function(doc, selection, reason) {}
};

var DocumentActionListener = {
  buffer : "",
  WillCreateNode: function(tag, parent, position) {},
  DidCreateNode: function(tag, node, parent, position, result) {},
  WillInsertNode: function(node, parent, position)
  {
    /*if (node.nodeType == 3)
      if (node.nodeValue.length == 1)
        this.buffer += node.nodeValue;*/
  },
  DidInsertNode: function(node, parent, position, result) {},
  WillDeleteNode: function(child) {},
  DidDeleteNode: function(child, result) {},
  WillSplitNode: function(existingRightNode, offset) {},
  DidSplitNode: function(existingRightNode, offset, newLeftNode, result) {},
  WillJoinNodes: function(leftNode, rightNode, parent) {},
  DidJoinNodes: function(leftNode, rightNode, parent, result) {},
  WillInsertText: function(textNode, offset, string)
  {
    fileModified = true;
    //editorHandle.currentStatus(string, 100);
    /*if (string == " ") {
      var ds           = rdfStore.getSource("blogHistory");
      alert("<"+this.buffer+">");
      var result       = ds.GetSource(RDF.GetResource("http://www.exitspace.net/blog/postid"),
                                      RDF.GetLiteral(this.buffer), true);
      this.buffer = "";
      alert (result.Value);
    } else {
      if (string.length == 1)
        this.buffer += string;
    }*/
  },
  DidInsertText: function(textNode, offset, string, result) {},
  WillDeleteText: function(textNode, offset, length) {},
  DidDeleteText: function(textNode, offset, length, result)
  {
    //alert(this.buffer);
  },
  WillDeleteSelection: function(selection) {},
  DidDeleteSelection: function(selection)
  {
    //this.buffer = this.buffer.slice(0, -1);
  }
};
